$ Title Hybrid Top-down Computable General Equilibrium and Bottom-up Electricity Expansion Model
$ontext
--------------------------------------------------------------------------------
------------------------------ General Header ----------------------------------
--------------------------------------------------------------------------------
         Hybrid Top-down Computable General Equilibrium and Bottom-up Electricity Expansion Model
         Renato Rodrigues
         Instituto de Investigacin Tecnolgica
         Date: 13, Sep, 2011
--------------------------------------------------------------------------------
--------------------------- End of General Header ------------------------------
--------------------------------------------------------------------------------
$offtext

*-------------------------------------------------------------------------------
*------------------------------------- Options ---------------------------------
*-------------------------------------------------------------------------------
$include ./Auxiliar_Files/options.gms

Option Reslim=100000;

* Enable export results to excel files
* 1 = ON / 0 = OFF
$set excel_export 1

*-------------------------------------------------------------------------------
*-------------------------------------- Sets -----------------------------------
*-------------------------------------------------------------------------------
*Including macros
$include ./Auxiliar_Files/macros.gms

*Defining sets
$include ./Auxiliar_Files/sets.gms

*-------------------------------------------------------------------------------
*----------------------------------- Parameters --------------------------------
*-------------------------------------------------------------------------------
*Defining Exogenous Parameters
$include ./Auxiliar_Files/exogenous_parameters.gms

*-------------------------------------------------------------------------------
*------------------------ Choose the model to be executed ----------------------
*-------------------------------------------------------------------------------
*Choose model to be solved:
$if not set Model $set Model A_4
*Choose the load block aggregation (Change it in GEMED.exe. It shouldnt be changed in gams file)
$Set Load_Block_Aggregation LB_180
*Choose the calibration aproximation method (MinMax, MinMaxPlus, Quadratic):
$Set Calibration_Problem Quadratic
*Choose the years to be simulated in the electricity operation and investment model:
$Set Years_Simulated 2005-2020


*Choose the Bottom-up Electricity model type:
*$if not set exec_BU_Linear $Set exec_BU_Linear 1
*$if not set exec_NLP $Set exec_BU_MCP 0


set model_description /
*PURE MODELS
         1 Top-down Pure CGE Model
         2 Bottom-up Electricity Operation and Expansion Model
         3 SAM disaggregation in load blocks and technologies
         4 GEMED - Electricity Extended Top-down Pure CGE Model
         5 Iterative SAM disaggregation and electricity model results compatibility
         6 Bottom-up Electricity Expansion Mixed Complementarity Model
         7 H-GEMED - Hybrid Top-down CGE Bottom-up Electricity Expansion Model
*         8 TO BE IMPLEMENTED - D-H-GEMED - Recursive Hybrid Top-down CGE Bottom-up Expansion Model
*         9 TO BE IMPLEMENTED - Benders-H-GEMED - Hybrid Top-down CGE Bottom-up Electricity Benders Decomposed Expansion Model (GEMED+operation primal and investments dual)
*         9 TO BE IMPLEMENTED - D-GEMED - Recursive Top-down CGE Bottom-up Expansion Model
*         10 TO BE IMPLEMENTED - BU-and-GEMED - BU Electricity load block independent linked with TD GEMED Model
*         11 TO BE IMPLEMENTED - BU-and-D-GEMED - BU Electricity load block independent linked with TD D-GEMED Model
*         12 TO BE IMPLEMENTED - feedback-BU-and-D-GEMED - BU Electricity load block independent linked with TD D-GEMED Model with feedback
*POLICY EVALUATIONS
*A - Active Demand Management policies (ADM) (default policy with demand response displacement and conservation measures, change the bellow options to alternative evaluation)
         A_1 Active Demand Management in the pure Bottom-up Electricity Expansion Lineal Model
         A_2 Active Demand Management in the pure Bottom-up Electricity Expansion Lineal Model (with rolling horizon operation and expansion decisions)
         A_3 Active Demand Management in the pure CGE Model
         A_4 Active Demand Management in the GEMED Model with a single SAM calibration
         A_5 Active Demand Management in the GEMED Model with a iterative calibration
         A_6 TO BE IMPLEMENTED - Active Demand Management in the H-GEMED Model
*B - Allocation of Renewable Subsidies (ARS) - Evaluation of alternative fiscal instruments and economical consequences
         /;

$if %Calibration_Problem%==MinMax $Set Abreviated_calibration_problem MM
$if %Calibration_Problem%==MinMaxPlus $Set Abreviated_calibration_problem MMP
$if %Calibration_Problem%==Quadratic $Set Abreviated_calibration_problem Q


*Creating resumed model execution log file (Executions_log.txt)
File Execution_log /Execution.log/;
put Execution_log;
*Executions_log.ap = 1;
put model_description.te("%Model%") //;
put @5 'Execution date and time...............      'system.DATE ' ' system.TIME  /;
put @5 'Model Code............................      %Model%' /;
put @5 'Load block aggregation used...........      %Load_Block_Aggregation%' /;
put @5 'Calibration problem objective.........      %Calibration_Problem%' /;
put @5 'Years simulated.......................      %Years_Simulated%' //;

$If "%Model%" == "1" $goto Model_1
$If "%Model%" == "2" $goto Model_2
$If "%Model%" == "3" $goto Model_3
$If "%Model%" == "4" $goto Model_4
$If "%Model%" == "5" $goto Model_5
$If "%Model%" == "6" $goto Model_6
$If "%Model%" == "7" $goto Model_7
$If "%Model%" == "A_1" $goto Model_A_1
$If "%Model%" == "A_2" $goto Model_A_2
$If "%Model%" == "A_3" $goto Model_A_3
$If "%Model%" == "A_4" $goto Model_A_4
$If "%Model%" == "A_5" $goto Model_A_5
$If "%Model%" == "A_6" $goto Model_A_6



*-------------------------------------------------------------------------------
*------------------------------------- Models ----------------------------------
*-------------------------------------------------------------------------------

*-------------------------------------------------------------------------------
*--------------------------- 1 = Top-down Pure CGE Model -----------------------
*-------------------------------------------------------------------------------
$label Model_1

*-----------CGE MODEL EXECUTION
$include ./Models/Model_1_TD_Pure_CGE_Model.gms
         if (exec_MCP=1, SOLVE CGE_MCP using MCP;execution_log_file(CGE_MCP));
         if (exec_NLP=1, SOLVE CGE_NLP minimizing DUMMY_CGE using nlp;execution_log_file(CGE_NLP));

*        returning parameters and variables to original non scaled values
CGE_scale_parameters(*)
CGE_scale_results(*)

$goto end
*-------------------------------------------------------------------------------
*-------------- 2 = Bottom-up Electricity Expansion Lineal Model ---------------
*-------------------------------------------------------------------------------
$label Model_2
*-----------ELECTRICITY MODEL EXECUTION
$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
* lineal model solution
         if (exec_BU_LP=1, SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU));
* start point solution
         if (exec_BU_MCP=1, SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP;);
* mcp model solution
         if (exec_BU_MCP=1, SOLVE MCPGEN_BAU using MCP;);

*         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP;
*         SOLVE MCPGEN using MCP;

$goto end
*-------------------------------------------------------------------------------
*----------- 3 = SAM disaggregation in load blocks and technologies ------------
*-------------------------------------------------------------------------------
$label Model_3

*-----------ELECTRICITY MODEL EXECUTION
$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        sending optimization results as parameters to the following model
         parameters energy_only_price_SAMDIS(y,l,dp,db), pgen_SAMDIS(y,t,f,l,gp,gb), ppumped_SAMDIS(y,l,gp,gb), BU_mkt_surplus(y,l,p,b), BU_dist_factor(y,l,p,b), BU_market_failures_and_non_accounted_costs(y,l,p,b);
         Send_Results_LEGEN_BAU_to_SAMDIS

*-----------SAM DISAGGREGATION MODEL EXECUTION
$include ./Models/Model_3_SAM_disaggregation_in_load_blocks.gms
         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS;execution_log_file(SAMDIS));
         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS;execution_log_file(NON_LINEAL_SAMDIS));

*        exporting results
$if "%excel_export%" == "1" $include ./Auxiliar_Output_Files/Model_3_SAM_disaggregation_output.gms

$goto end
*-------------------------------------------------------------------------------
*---------- 4 = GEMED - Electricity Extended Top-down Pure CGE Model -----------
*-------------------------------------------------------------------------------
$label Model_4

*-----------ELECTRICITY MODEL EXECUTION
$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        sending optimization results as parameters to the following model
         parameters energy_only_price_SAMDIS(y,l,dp,db), pgen_SAMDIS(y,t,f,l,gp,gb), ppumped_SAMDIS(y,l,gp,gb), BU_mkt_surplus(y,l,p,b), BU_dist_factor(y,l,p,b), BU_market_failures_and_non_accounted_costs(y,l,p,b);
         Send_Results_LEGEN_BAU_to_SAMDIS

*-----------SAM DISAGGREGATION MODEL EXECUTION
$include ./Models/Model_3_SAM_disaggregation_in_load_blocks.gms
         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS; execution_log_file(SAMDIS));
         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS; execution_log_file(NON_LINEAL_SAMDIS));

*        sending optimization results as parameters to the following model
         Send_Results_SAMDIS_to_GEMED

*-----------EXTENDEND CGE MODEL EXECUTION
$include ./Models/Model_4_GEMED_Electricity_Extended_TD_Pure_CGE_Model.gms
         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
GEMED_scale_parameters(*)
GEMED_scale_results(*)

*        exporting results
$if "%excel_export%" == "1" $include ./Auxiliar_Output_Files/Model_3_SAM_disaggregation_output.gms

$goto end
*-------------------------------------------------------------------------------
*- 5 = Iterative SAM disaggregation and electricity model results compatibility -
*-------------------------------------------------------------------------------
$label Model_5

*-----------GENERAL OPTIONS to define maximun number of iterations and solution tolerance
         set     iter(iterations) max number of iterations        /1*50/
                 iter_executed(iterations);

         scalar  end_loop        auxiliar exit loop scalar       /0/
                 tol             percentage tolerance            /1e-14/;

*-----------STARTING EXECUTION

*-----------FIRST ITERATION
         iter_executed("1")=yes;

*        ELECTRICITY MODEL EXECUTION
$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        sending optimization results as parameters to the following model
         parameters energy_only_price_SAMDIS(y,l,dp,db), pgen_SAMDIS(y,t,f,l,gp,gb), ppumped_SAMDIS(y,l,gp,gb), BU_mkt_surplus(y,l,p,b), BU_dist_factor(y,l,p,b), BU_market_failures_and_non_accounted_costs(y,l,p,b);
         Send_Results_LEGEN_BAU_to_SAMDIS

*        SAM DISAGGREGATION MODEL EXECUTION
$include ./Models/Model_3_SAM_disaggregation_in_load_blocks.gms
         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS; execution_log_file(SAMDIS) );
         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS; execution_log_file(NON_LINEAL_SAMDIS) );

*        Reseting (initialising) parameters
         Deviations_by_iteration

*        Saving SAM disaggregation iteration results
         Iteration_results

*-----------REMAINING EXECUTIONS
         loop (iter $ ( (end_loop=0) and (iter.val>1) ) ,
                         iter_executed(iter)=yes;

*                        sending optimization results as parameters to the following model
                         Send_Results_SAMDIS_to_LEGEN

*                        ELECTRICITY MODEL EXECUTION
                         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*                        sending optimization results as parameters to the following model
                         Send_Results_LEGEN_BAU_to_SAMDIS

*                        SAM DISAGGREGATION MODEL EXECUTION
                         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS; execution_log_file(SAMDIS) );
                         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS; execution_log_file(NON_LINEAL_SAMDIS) );

*                         reseting parameters
                         Deviations_by_iteration

*                        auxiliar parameters for results evaluation
                         Iteration_results

*                        exiting loop
                         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, if( (max_deviation(iter) < tol), end_loop=1 ); );
                         if (exec_Quadratic=1, if( (max(oem_vom_dev(iter),oem_fom_labor_dev(iter),oem_fom_social_contribution_dev(iter),oem_fom_equipments_dev(iter),thermodynamic_efficiency_dev(iter),own_cons_gen_pctg_dev(iter),overnight_costs_dev(iter),tdeo_losses_pctg_dev(iter),exp_price_pctg_dev(iter),imp_price_pctg_dev(iter),tdeo_pf_pct_adj_dev(iter),co2e_content_dev(iter)) < tol), end_loop=1 ););
         );

*        ploting results
$if "%excel_export%" == "1" $include ./Auxiliar_Output_Files/Model_3_SAM_disaggregation_output.gms
$if "%excel_export%" == "1" $include ./Auxiliar_Output_Files/Model_5_Iterative_SAM_disaggregation_output.gms

$goto end
*-------------------------------------------------------------------------------
*------- 6 = Bottom-up Electricity Expansion Mixed Complementarity Model -------
*-------------------------------------------------------------------------------
$label Model_6

$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

execute_unload 'LEGEN_BAU.gdx'

*-----------ELECTRICITY MODEL EXECUTION
         SOLVE MCPGEN_BAU using MCP; execution_log_file(LEGEN_BAU)

FUEL_COST_LEGEN.l(eys,t,f,l,p,b) = PGEN_LEGEN.l(eys,t,f,l,p,b)*thermodynamic_efficiency(eys,t,f)*fuel_price(eys,t,f)/1e6;
CO2_PAYMENTS_LEGEN.l(eys,t,f,l,p,b) = PGEN_LEGEN.l(eys,t,f,l,p,b)*co2e_content(f,t)/1e6*co2_price(eys);
VOM_LEGEN.l(eys,t,f,l,p,b) = init_oem_vom(eys,t)*PGEN_LEGEN.l(eys,t,f,l,p,b)/1e6;
INSTALLED_CAPACITY_LEGEN.l(eys,l,t)= existent_installed_capacity(eys,l,t) + sum((eys_alias)$((ord(eys_alias) le ord(eys)) and (ord(eys_alias) gt (ord(eys)-power_plant_time_life(t)))),PINS_LEGEN.l(eys_alias,l,t));
AMORTIZATION_LEGEN.l(eys,l,t)= init_overnight_costs(eys,t) * IDC(t) * capital_recovery_factor(t) * ( existent_installed_capacity_to_be_amortized(eys,l,t) +  sum((eys_alias)$((ord(eys_alias) le ord(eys)) and (ord(eys_alias) gt (ord(eys)-power_plant_time_life(t)))),PINS_LEGEN.l(eys_alias,l,t)) )/1e3;


execute_unload 'MCPGEN_BAU.gdx'

$goto end

*-------------------------------------------------------------------------------
*--- 7 = H-GEMED - Hybrid Top-down CGE Bottom-up Electricity Expansion Model ---
*-------------------------------------------------------------------------------
$label Model_7




$goto end
*-------------------------------------------------------------------------------
*--- 8 = D-H-GEMED - Recursive Hybrid Top-down CGE Bottom-up Expansion Model ---
*-------------------------------------------------------------------------------
$label Model_8







$goto end
*-------------------------------------------------------------------------------





*-------------------------------------------------------------------------------
*------------------------------ POLICY EVALUATIONS -----------------------------
*-------------------------------------------------------------------------------



*-------------------------------------------------------------------------------
*--------------------- A - Active Demand Management policies -------------------
*-------------------------------------------------------------------------------


*-------------------------------------------------------------------------------
*----- A_1 = ADM in the pure Bottom-up Electricity Expansion Lineal Model ------
*-------------------------------------------------------------------------------
$label Model_A_1

if( not(exec_DR_displac=1 or exec_DR_reduc=1), exec_DR_displac=1; exec_DR_reduc=1 );

*-----------MODEL EXECUTION

*        ELECTRICITY MODEL EXECUTION (Initial execution in order to define gad initial prices)
$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        initial reference price to GAD
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;

*        BAU results
         LEGEN_results(BAU)

*-----------FINAL EXECUTION
         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN)

*        Case study results
         LEGEN_results(Case_study)

*        comparing results
         LEGEN_results_comparison

*        exporting results
$if "%excel_export%" == "1" $include ./Auxiliar_Output_Files/Model_2_LEGEN_results_comparison_output.gms

*        output DR policy assessment results to excel
$if "%excel_export%" == "1" $include ./Auxiliar_output_files/LEGEN_results_comparison_output.gms


$goto end
*-------------------------------------------------------------------------------
*----- A_2 = ADM in the pure Bottom-up Electricity Expansion Lineal Model ------
*---------- (with rolling horizon operation and expansion decisions)  ----------
*-------------------------------------------------------------------------------
$label Model_A_2

if( not(exec_DR_displac=1 or exec_DR_reduc=1), exec_DR_displac=1; exec_DR_reduc=1 );

*-----------GENERAL OPTIONS to define maximun number of iterations and the ADM rolling horizon length
         sets    iter                                            /1*100/
                 subiter(iter)   'rolling horizon iteration'
                 fixed(iter,eys);
         parameter number_of_years_in_iter                       /5/;

         loop((iter,eys), if(iter.val lt (ord(eys)/number_of_years_in_iter), subiter(iter) = yes; ));
         loop((subiter,eys), if( ( (ord(eys) gt (subiter.val-1)*number_of_years_in_iter) and (ord(eys) lt (subiter.val*number_of_years_in_iter)+1) ), fixed(subiter,eys) = yes; ) );

*-----------MODEL EXECUTION

*        ELECTRICITY MODEL EXECUTION (Initial execution in order to define gad initial prices)
$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
*         LEGEN.tolinfeas = 1e-7;
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        initial reference price to GAD
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;

*        BAU results
         LEGEN_results(BAU)

*        CASE STUDY EXECUTION
*        Initial execution (ALL YEARS EVALUATED)
         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN)
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;

*-----------ROLLING HORIZON LOOP FOR ADM EVALUATION
         loop(subiter,
                 loop(eys$fixed(subiter,eys),
                         PGEN_LEGEN.fx(eys,t,f,l,p,b) = PGEN_LEGEN.l(eys,t,f,l,p,b);
                         PPUMPED_LEGEN.fx(eys,l,p,b) = PPUMPED_LEGEN.l(eys,l,p,b);
                         RES_LEGEN.fx(eys,l,p) = RES_LEGEN.L(eys,l,p);
                         PINS_LEGEN.fx(eys,l,t) = PINS_LEGEN.l(eys,l,t);
                         INCREASED_DR_LOAD.fx(eys,l,p,b)=INCREASED_DR_LOAD.l(eys,l,p,b);
                         DECREASED_DR_LOAD.fx(eys,l,p,b)=DECREASED_DR_LOAD.l(eys,l,p,b);
                         CONSERVED_DR_LOAD.fx(eys,l,p,b)=CONSERVED_DR_LOAD.l(eys,l,p,b);
                     );

*                ELECTRICITY MODEL EXECUTION (FIXING PREVIOUS YEARS RESULTS)
                 SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN)
                 GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;
         );

*-----------FINAL EXECUTION
         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN)

*        Case study results
         LEGEN_results(Case_study)

*        comparing results
         LEGEN_results_comparison

*        exporting results
$if "%excel_export%" == "1" $include ./Auxiliar_Output_Files/Model_2_LEGEN_results_comparison_output.gms

$goto end
*-------------------------------------------------------------------------------
*---------------------- A_3 = ADM in the pure CGE Model ------------------------
*-------------------------------------------------------------------------------
$label Model_A_3

if( not(exec_DR_displac=0 and exec_DR_reduc=1), exec_DR_displac=0; exec_DR_reduc=1 );

*-----------ELECTRICITY MODEL EXECUTION
$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        initial reference price to GAD
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;

*-----------CGE MODEL EXECUTION
$include ./Models/Model_1_TD_Pure_CGE_Model.gms
         if (exec_MCP=1, SOLVE CGE_MCP using MCP; execution_log_file(CGE_MCP));
         if (exec_NLP=1, SOLVE CGE_NLP minimizing DUMMY_CGE using nlp; execution_log_file(CGE_NLP));

*        returning parameters and variables to original non scaled values
         CGE_scale_parameters(*)
         CGE_scale_results(*)

*        BAU results
         CGE_results(BAU)

*-----------ELECTRICITY MODEL EXECUTION
         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; abort $ (LEGEN.modelstat > 2) "No optimal solution found";

         Q_H_D_CGE.fx(ey,"Electricity")= ( Q_H_D_CGE.l(ey,"Electricity") * ( 1 - ( sum((l,b,p),CONSERVED_DR_LOAD.l(ey,l,p,b)) / sum((l,b,p),demand(ey,"Households",l,p,b) ) ) ) ) / scale_factor;

*        scaling parameters
         CGE_scale_parameters(/)

*-----------CASE STUDY EXECUTION
         if (exec_MCP=1, SOLVE CGE_MCP using MCP; execution_log_file(CGE_MCP));
         if (exec_NLP=1, SOLVE CGE_NLP minimizing DUMMY_CGE using nlp; execution_log_file(CGE_NLP));

*        returning parameters and variables to original non scaled values
         CGE_scale_parameters(*)
         CGE_scale_results(*)

*        case study results
         CGE_results(Case_study)

*        comparing results
         CGE_results_comparison

*        output results to excel
$if "%excel_export%" == "1" $include ./Auxiliar_output_files/Model_1_CGE_results_comparison_output.gms

$goto end
*-------------------------------------------------------------------------------
*--------- A_4 = ADM in the GEMED Model with a single SAM calibration ----------
*-------------------------------------------------------------------------------
$label Model_A_4

if( not(exec_DR_displac=1 or exec_DR_reduc=1), exec_DR_displac=1; exec_DR_reduc=1 );

*Makes more sense to use the MinMax calibration with only one calibration round
*exec_MinMax=0; exec_MinMax_plus_lineal_term=1; exec_Quadratic=0;

*-----------ELECTRICITY MODEL EXECUTION
$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        sending optimization results as parameters to the following model
         parameters energy_only_price_SAMDIS(y,l,dp,db), pgen_SAMDIS(y,t,f,l,gp,gb), ppumped_SAMDIS(y,l,gp,gb), BU_mkt_surplus(y,l,p,b), BU_dist_factor(y,l,p,b), BU_market_failures_and_non_accounted_costs(y,l,p,b);
         Send_Results_LEGEN_BAU_to_SAMDIS

*        initial reference price to GAD
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;

*-----------SAM DISAGGREGATION MODEL EXECUTION
$include ./Models/Model_3_SAM_disaggregation_in_load_blocks.gms
         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS; execution_log_file(SAMDIS));
         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS; execution_log_file(NON_LINEAL_SAMDIS));

*        sending optimization results as parameters to the following model
         Send_Results_SAMDIS_to_GEMED

*-----------EXTENDEND CGE MODEL EXECUTION
$include ./Models/Model_4_GEMED_Electricity_Extended_TD_Pure_CGE_Model.gms
         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(*)
         GEMED_scale_results(*)

*        BAU results
         LEGEN_results(BAU)
         GEMED_CGE_results(BAU)
         DR_energy_economics_results(BAU)

*-----------CASE STUDY EXECUTION
*-----------ELECTRICITY MODEL EXECUTION
         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN)

*        fixing DR policy consequences
         Q_H_D_GEN_GEMED.fx(ey,l,p,b) = ( Q_H_D_GEN_GEMED.l(ey,l,p,b) * ( 1 + ( ( (INCREASED_DR_LOAD.l(ey,l,p,b) - DECREASED_DR_LOAD.l(ey,l,p,b) )$(exec_DR_displac) - ( CONSERVED_DR_LOAD.l(ey,l,p,b) )$(exec_DR_reduc) ) / demand(ey,"Households",l,p,b) ) )) / scale_factor;

*        scale parameters
         GEMED_scale_parameters(/)

*-----------GEMED MODEL EXECUTION
*SOLVE GEMED_MCP_GAD using MCP; execution_log_file(GEMED_MCP)

         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(*)
         GEMED_scale_results(*)

*        case study results
         LEGEN_results(Case_study)
         GEMED_CGE_results(Case_study)
         DR_energy_economics_results(Case_study)

*        comparing results
         LEGEN_results_comparison
         GEMED_CGE_results_comparison
         GEMED_CGE_emissions_results_comparison
         DR_energy_economics_results_comparison

*        output calibration results
$if "%excel_export%" == "1" $include ./Auxiliar_Output_Files/Model_3_SAM_disaggregation_output.gms
*        output LEGEN, CGE and/or GEMED results to excel
$if "%excel_export%" == "1" $include ./Auxiliar_output_files/Model_4_GEMED_results_comparison_output.gms
*        output DR policy assessment results to excel
*$if "%excel_export%" == "1" $include ./Auxiliar_output_files/LEGEN_results_comparison_output.gms
*        output DR policy assessment results to excel
$if "%excel_export%" == "1" $include ./Auxiliar_output_files/Paper_energy_economics_DR_results_output.gms

$goto end
*-------------------------------------------------------------------------------
*---------- A_5 = ADM in the GEMED Model with a iterative calibration ----------
*-------------------------------------------------------------------------------
$label Model_A_5

if( not(exec_DR_displac=1 or exec_DR_reduc=1), exec_DR_displac=1; exec_DR_reduc=1 );

*-----------GENERAL OPTIONS to define maximun number of iterations and solution tolerance
         set     iter(iterations) max number of iterations        /1*50/
                 iter_executed(iterations);

         scalar  end_loop        auxiliar exit loop scalar       /0/
                 tol             percentage tolerance            /1e-14/;

*-----------STARTING EXECUTION

*-----------BUSINESS AS USUAL EXECUTION

*-----------FIRST ITERATION
         iter_executed("1")=yes;

*        ELECTRICITY MODEL EXECUTION
$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        sending optimization results as parameters to the following model
         parameters energy_only_price_SAMDIS(y,l,dp,db), pgen_SAMDIS(y,t,f,l,gp,gb), ppumped_SAMDIS(y,l,gp,gb), BU_mkt_surplus(y,l,p,b), BU_dist_factor(y,l,p,b), BU_market_failures_and_non_accounted_costs(y,l,p,b);
         Send_Results_LEGEN_BAU_to_SAMDIS

*        SAM DISAGGREGATION MODEL EXECUTION
$include ./Models/Model_3_SAM_disaggregation_in_load_blocks.gms
         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS; execution_log_file(SAMDIS) );
         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS; execution_log_file(NON_LINEAL_SAMDIS) );

*        Reseting (initialising) parameters
         Deviations_by_iteration

*        Saving SAM disaggregation iteration results
         Iteration_results

*-----------REMAINING EXECUTIONS
         loop (iter $ ( (end_loop=0) and (iter.val>1) ) ,
                         iter_executed(iter)=yes;

*                        sending optimization results as parameters to the following model
                         Send_Results_SAMDIS_to_LEGEN

*                        ELECTRICITY MODEL EXECUTION
                         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*                        sending optimization results as parameters to the following model
                         Send_Results_LEGEN_BAU_to_SAMDIS

*                        SAM DISAGGREGATION MODEL EXECUTION
                         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS; execution_log_file(SAMDIS) );
                         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS; execution_log_file(NON_LINEAL_SAMDIS) );

*                        reseting parameters
                         Deviations_by_iteration

*                        auxiliar parameters for results evaluation
                         Iteration_results

*                        exiting loop
                         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, if( (max_deviation(iter) < tol), end_loop=1 ); );
                         if (exec_Quadratic=1, if( (max(oem_vom_dev(iter),oem_fom_labor_dev(iter),oem_fom_social_contribution_dev(iter),oem_fom_equipments_dev(iter),thermodynamic_efficiency_dev(iter),own_cons_gen_pctg_dev(iter),overnight_costs_dev(iter),tdeo_losses_pctg_dev(iter),exp_price_pctg_dev(iter),imp_price_pctg_dev(iter),tdeo_pf_pct_adj_dev(iter),co2e_content_dev(iter)) < tol), end_loop=1 ););
         );


*        sending optimization results as parameters to the following model
         Send_Results_SAMDIS_to_LEGEN

*        ELECTRICITY MODEL EXECUTION (executing electricity model with final calibration solution)
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP ; execution_log_file(LEGEN_BAU)

*        initial reference price to GAD
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;

*        sending optimization results as parameters to the following model
         Send_Results_SAMDIS_to_GEMED

*        EXTENDEND CGE MODEL EXECUTION
$include ./Models/Model_4_GEMED_Electricity_Extended_TD_Pure_CGE_Model.gms
*GEMED_MCP.tolinfeas = 1e-2;
*GEMED_NLP.tolinfeas = 1e-2;
         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(*)
         GEMED_scale_results(*)

*        BAU results
         LEGEN_results(BAU)
         GEMED_CGE_results(BAU)

*-----------CASE STUDY EXECUTION
*        ELECTRICITY MODEL EXECUTION
         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN)

*        fixing DR policy consequences
         Q_H_D_GEN_GEMED.fx(ey,l,p,b) = ( Q_H_D_GEN_GEMED.l(ey,l,p,b) * ( 1 + ( ( (INCREASED_DR_LOAD.l(ey,l,p,b) - DECREASED_DR_LOAD.l(ey,l,p,b) )$(exec_DR_displac) - ( CONSERVED_DR_LOAD.l(ey,l,p,b) )$(exec_DR_reduc) ) / demand(ey,"Households",l,p,b) ) ) ) / scale_factor;

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(/)

*-----------CASE STUDY GEMED MODEL EXECUTION
         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(*)
         GEMED_scale_results(*)

*        case study results
         LEGEN_results(Case_study)
         GEMED_CGE_results(Case_study)

*        comparing results
         LEGEN_results_comparison
         GEMED_CGE_results_comparison
         GEMED_CGE_emissions_results_comparison

*        output results to excel
$if "%excel_export%" == "1" $include ./Auxiliar_output_files/Model_4_GEMED_results_comparison_output.gms
$if "%excel_export%" == "1" $include ./Auxiliar_Output_Files/Model_3_SAM_disaggregation_output.gms
$if "%excel_export%" == "1" $include ./Auxiliar_Output_Files/Model_5_Iterative_SAM_disaggregation_output.gms

$goto end
*-------------------------------------------------------------------------------
*----------------------- A_6 = ADM in the H-GEMED Model ------------------------
*-------------------------------------------------------------------------------
$label Model_A_6

if( not(exec_DR_displac=1 or exec_DR_reduc=1), exec_DR_displac=1; exec_DR_reduc=1 );

*-----------MODEL EXECUTION

*        ELECTRICITY MODEL EXECUTION (Initial execution in order to define gad initial prices)
$include ./Models/Model_2_BU_Electricity_Operation_and_Expansion_Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)
*         SOLVE MCPGEN_BAU using MCP; execution_log_file(LEGEN_BAU); execution_log_file(LEGEN_BAU)

*        initial reference price to GAD
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;

*        BAU results
         LEGEN_results(BAU)

*-----------FINAL EXECUTION
*         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN)

         execute_unload 'LEGEN.gdx'

parameter
total_increase(eys,l,p)
total_decrease(eys,l,p)
saldo(eys,l,p)
;

total_increase(eys,l,p) = sum(b, INCREASED_DR_LOAD.l(eys,l,p,b)*duration(l,p,b) );
total_decrease(eys,l,p) = sum(b, DECREASED_DR_LOAD.l(eys,l,p,b)*duration(l,p,b));
saldo(eys,l,p) =   total_increase(eys,l,p) - total_decrease(eys,l,p);

display  total_increase, total_decrease,  saldo;


*OPTION ITERLIM = 0;
         SOLVE MCPGEN_BAU using MCP;

execute_unload 'MCPGEN%ierationumber%.gdx'

;

total_increase(eys,l,p) = sum(b, INCREASED_DR_LOAD.l(eys,l,p,b)*duration(l,p,b) );
total_decrease(eys,l,p) = sum(b, DECREASED_DR_LOAD.l(eys,l,p,b)*duration(l,p,b));

saldo(eys,l,p) =   total_increase(eys,l,p) - total_decrease(eys,l,p);

display  total_increase, total_decrease,  saldo;

$goto end
*-------------------------------------------------------------------------------





$label end

