$Title Electricity Model
$ontext
--------------------------------------------------------------------------------
------------------------------ General Header ----------------------------------
--------------------------------------------------------------------------------
         Bottom-up Electricity Expansion Lineal Model
         Renato Rodrigues
         Instituto de Investigacin Tecnolgica
         Date: Abril, 2011
         Models in this file:
                 LEGEN_BAU - linear operation and investiment electricity model
                 LEGEN - linear operation and investiment electricity model with Demand Response policy
                 MCPGEN_BAU - mixed complementarity operation and investiment electricity model
                 MCPGEN - mixed complementarity operation and investiment electricity model with Demand Response policy
--------------------------------------------------------------------------------
--------------------------- End of General Header ------------------------------
--------------------------------------------------------------------------------
$offtext

*-------------------------------------------------------------------------------
*------------------------------------- Sets ------------------------------------
*-------------------------------------------------------------------------------
*Sets defined in sets.gms
alias (eys,eys_alias);

*-------------------------------------------------------------------------------
*---------------------------------- Parameters ---------------------------------
*-------------------------------------------------------------------------------
*Exogenously defined parameters in exogenous_parameters.gms
*Without a additional restriction the water management is much more used than in real world operation. Hyd Res maximum production was 2745MW, which corresponds to a 50% only availability. This term adjusts the availability to reflect the real world operation.
parameter stochastic_adjusted_availability(eys,l,p,b,t);
stochastic_adjusted_availability(eys,l,p,b,t)=1;
stochastic_adjusted_availability(eys,"Peninsular",p,b,"Hyd_Res")=Hyd_Res_stochastic_water_adjusted_availability(eys,"Peninsular",b,p);


* Penalize unnecessary displacements to avoid multiple solutions (1=true, 0=false)
parameter penalize_displacements;
penalize_displacements=1;

*-------------------------------------------------------------------------------
*---------------------------------- Variables ----------------------------------
*-------------------------------------------------------------------------------
Variables
PGEN_LEGEN(y,t,f,l,p,b)          MW              Electricity power generation by each technology
PPUMPED_LEGEN(y,l,p,b)           MW              Pumping consumed electricity power
RES_LEGEN(y,l,p)                 MW              Hydro technology reservoir level
VOM_LEGEN(y,t,f,l,p,b)           millions 
FUEL_COST_LEGEN(y,t,f,l,p,b)     millions 
CO2_PAYMENTS_LEGEN(y,t,F,l,p,b)  millions 
TOTAL_GENERATION_COST_LEGEN(y,l) millions 
PINS_LEGEN(y,l,t)                MW              New capacity instaled potency
INSTALLED_CAPACITY_LEGEN(y,l,t)  MW              Total capacity instaled potency
AMORTIZATION_LEGEN(y,l,t)        millions 
OBJECTIVE_LEGEN                  millions 
* Demand response
INCREASED_DR_LOAD(y,l,p,b)       MW              Increased demand in demand reponse load displacements
DECREASED_DR_LOAD(y,l,p,b)       MW              Reduced demand in demand reponse load displacements
CONSERVED_DR_LOAD(y,l,p,b)       MW              Conserved demand
;

Positive Variables
PGEN_LEGEN, PPUMPED_LEGEN, RES_LEGEN, VOM_LEGEN, FUEL_COST_LEGEN, TOTAL_GENERATION_COST_LEGEN, CO2_PAYMENTS_LEGEN,
PINS_LEGEN, INSTALLED_CAPACITY_LEGEN, AMORTIZATION_LEGEN,INCREASED_DR_LOAD, DECREASED_DR_LOAD, CONSERVED_DR_LOAD
;

*Reservoir Intra year management
RES_LEGEN.fx(eys,l,p1)=res_init(eys,l,"Hyd_Res");

RES_LEGEN.up(eys,l,p) = res_max(eys,l,"Hyd_Res");

*-------------------------------------------------------------------------------
*----------------------------- Defining Model Equations ------------------------
*-------------------------------------------------------------------------------
Equations
eq_Total_generation_cost(eys,l)
eq_demand_balance_BAU(eys,l,p,b)
eq_hydro_reservoir_level(eys,l,p)
eq_pumping_efficiency(eys,l)
eq_maximal_pumping_production(eys,l)
eq_maximum_hydro_reservoir_level(eys,l,p)
eq_run_of_river_production(eys,l,p,b)
eq_fixed_FG_combustible_use(eys,l,p,b)
eq_VOM(eys,t,f,l,p,b)
eq_fuel_cost(eys,t,f,l,p,b)
eq_CO2_emission_costs(eys,t,f,l,p,b)
eq_pins_acum(eys,l,t)
eq_amortization(eys,l,t)
eq_pins_availability(eys,t,f,l,p,b)
eq_pgen_wind(eys,l,p,b)
eq_pgen_orsr(eys,l,p,b)
eq_reserves(eys,l)
eq_objective_function
* Demand response
eq_demand_balance_case_study(eys,l,p,b)
eq_maximum_displacement(eys,l,p,b)
eq_displacement_balance(eys,l,p)
eq_potency_conservation_limit(eys,l,p,b)
eq_minimal_savings_requirement(eys,l,p)
;

eq_objective_function..
OBJECTIVE_LEGEN =e= sum((eys,l), TOTAL_GENERATION_COST_LEGEN(eys,l));

eq_Total_generation_cost(eys,l)..
TOTAL_GENERATION_COST_LEGEN(eys,l) =e=
           exec_fc*(sum((t,f,p,b)$thermal_t(t,f), FUEL_COST_LEGEN(eys,t,f,l,p,b)*duration(l,p,b)))
         + exec_co2*(sum((t,f,p,b)$emitting_t(t,f), CO2_PAYMENTS_LEGEN(eys,t,f,l,p,b)*duration(l,p,b)))
         + exec_vom*(sum((t,f,p,b)$t_f(t,f), VOM_LEGEN(eys,t,f,l,p,b)*duration(l,p,b)))
         - exec_renew_prem*(sum((t,f,p,b)$renewable_premium_t(t,f), PGEN_LEGEN(eys,t,f,l,p,b)*duration(l,p,b)*renewable_premium(eys,t)/1e6))
         + exec_oem_labor*(sum((t), init_oem_fom_labor(eys,l,t)*INSTALLED_CAPACITY_LEGEN(eys,l,t) / 1e3))
         + exec_oem_sc*(sum((t), init_oem_fom_social_contribution(eys,l,t)*INSTALLED_CAPACITY_LEGEN(eys,l,t) / 1e3))
         + exec_oem_equip*(sum((t), init_oem_fom_equipments(eys,l,t)*INSTALLED_CAPACITY_LEGEN(eys,l,t) / 1e3))
         + exec_capac*(sum((t), AMORTIZATION_LEGEN(eys,l,t)))
         - exec_emiss_rights*( sum((t), co2_emission_rights_assignment_GEN_technology(eys,t)*co2_price(eys) ) )
         + penalize_displacements*(sum((p,b),INCREASED_DR_LOAD(eys,l,p,b))/1e6)
         ;

eq_demand_balance_BAU(eys,l,p,b)$duration(l,p,b)..
         sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b)) + pimported(eys,l,p,b) - PPUMPED_LEGEN(eys,l,p,b)
         - init_own_cons_gen_pctg*sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b))
         - tdeo_losses_pctg(l,p,b)*( sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b)) + pimported(eys,l,p,b) - PPUMPED_LEGEN(eys,l,p,b) + demand(eys,"Exports",l,p,b)  )
         =e= demand(eys,"households",l,p,b) + sum(sne, demand(eys,sne,l,p,b)) + demand(eys,"Exports",l,p,b);

eq_fuel_cost(eys,t,f,l,p,b)$duration(l,p,b)..
FUEL_COST_LEGEN(eys,t,f,l,p,b) =e= PGEN_LEGEN(eys,t,f,l,p,b)*thermodynamic_efficiency(eys,t,f)*fuel_price(eys,t,f)/1e6;

eq_CO2_emission_costs(eys,t,f,l,p,b)$duration(l,p,b)..
CO2_PAYMENTS_LEGEN(eys,t,f,l,p,b) =E= PGEN_LEGEN(eys,t,f,l,p,b)*co2e_content(f,t)/1e6*co2_price(eys);

eq_VOM(eys,t,f,l,p,b)$duration(l,p,b)..
VOM_LEGEN(eys,t,f,l,p,b) =E= init_oem_vom(eys,t)*PGEN_LEGEN(eys,t,f,l,p,b)/1e6;

eq_pins_acum(eys,l,t)..
INSTALLED_CAPACITY_LEGEN(eys,l,t)=e= existent_installed_capacity(eys,l,t) + sum((eys_alias)$((ord(eys_alias) le ord(eys)) and (ord(eys_alias) gt (ord(eys)-power_plant_time_life(t)))),PINS_LEGEN(eys_alias,l,t));
*INSTALLED_CAPACITY_LEGEN(eys,l,t)=e= existent_installed_capacity(eys,l,t) + sum((eys_alias)$((ord(eys_alias) le ord(eys)) and (ord(eys_alias) gt (ord(eys)-power_plant_time_life(t)))),PINS_LEGEN(eys_alias,l,t));


eq_amortization(eys,l,t)..
AMORTIZATION_LEGEN(eys,l,t)=e= init_overnight_costs(eys,t) * IDC(t) * capital_recovery_factor(t) * ( existent_installed_capacity_to_be_amortized(eys,l,t) +  sum((eys_alias)$((ord(eys_alias) le ord(eys)) and (ord(eys_alias) gt (ord(eys)-power_plant_time_life(t)))),PINS_LEGEN(eys_alias,l,t)) )/1e3;

eq_hydro_reservoir_level(eys,l,p)..
sum((b),PGEN_LEGEN(eys,"Hyd_Res","na",l,p,b)*duration(l,p,b)) - RES_LEGEN(eys,l,p) + RES_LEGEN(eys,l,p+1)$(ORD(p) lt card(p)) + res_init(eys+1,l,"Hyd_Res")$(ORD(p) EQ card(p)) =L= hydro_inflows(eys,l,p,"Hyd_Res");

eq_maximum_hydro_reservoir_level(eys,l,p)..
RES_LEGEN(eys,l,p) - res_max(eys,l,"Hyd_Res") =L= 0;

eq_run_of_river_production(eys,l,p,b)$duration(l,p,b)..
PGEN_LEGEN(eys,"Hyd_RoR","na",l,p,b) =E= hydro_inflows(eys,l,p,"Hyd_RoR")/sum((b_alias),duration(l,p,b_alias)) ;

eq_pins_availability(eys,t,f,l,p,b)$duration(l,p,b)..
PGEN_LEGEN(eys,t,f,l,p,b) - INSTALLED_CAPACITY_LEGEN(eys,l,t)*availability_factor(eys,l,t)*stochastic_adjusted_availability(eys,l,p,b,t)  =L=  0;

eq_pumping_efficiency(eys,l)..
sum((p,b), duration(l,p,b)*(PGEN_LEGEN(eys,"Pump","na",l,p,b) - PPUMPED_LEGEN(eys,l,p,b)*pump_efficiency) ) =L= 0;

eq_maximal_pumping_production(eys,l)..
sum((p,b), PGEN_LEGEN(eys,"Pump","na",l,p,b)*duration(l,p,b)) =L= res_max(eys,l,"Pump");

eq_fixed_FG_combustible_use(eys,l,p,b)$duration(l,p,b)..
PGEN_LEGEN(eys,"F-G","Fuel-oil",l,p,b)-pctg_fueloil_use_on_fg(eys,l)*sum((f),PGEN_LEGEN(eys,"F-G",f,l,p,b)) =e= 0;

eq_pgen_wind(eys,l,p,b)$duration(l,p,b)..
PGEN_LEGEN(eys,"Wind","na",l,p,b) =e= pgenerated("2005",l,p,b,"Wind") / existent_installed_capacity("2005",l,"Wind") * INSTALLED_CAPACITY_LEGEN(eys,l,"Wind");

eq_pgen_orsr(eys,l,p,b)$duration(l,p,b)..
PGEN_LEGEN(eys,"ORSR","na",l,p,b) =e= pgenerated("2005",l,p,b,"ORSR") / existent_installed_capacity("2005",l,"ORSR") * INSTALLED_CAPACITY_LEGEN(eys,l,"ORSR");

eq_reserves(eys,l)..
sum((t,f)$t_not_intermittent(t), INSTALLED_CAPACITY_LEGEN(eys,l,t)) =G= not_internittent_coverage_rate * smax((p,b), sum(sam,demand(eys,sam,l,p,b)) );

* Demand response
eq_demand_balance_case_study(eys,l,p,b)$duration(l,p,b)..
         sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b)) + pimported(eys,l,p,b) - PPUMPED_LEGEN(eys,l,p,b)
         - init_own_cons_gen_pctg*sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b))
         - tdeo_losses_pctg(l,p,b)*( sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b)) + pimported(eys,l,p,b) - PPUMPED_LEGEN(eys,l,p,b) + demand(eys,"Exports",l,p,b)  )
         =e= demand(eys,"households",l,p,b) + sum(sne, demand(eys,sne,l,p,b)) + demand(eys,"Exports",l,p,b)
         + ( INCREASED_DR_LOAD(eys,l,p,b) - DECREASED_DR_LOAD(eys,l,p,b) )$(exec_DR_displac) - ( CONSERVED_DR_LOAD(eys,l,p,b) )$(exec_DR_reduc) ;

eq_maximum_displacement(eys,l,p,b) $ (exec_DR_displac and duration(l,p,b))..
DECREASED_DR_LOAD(eys,l,p,b) =l= displaceable_load(eys,l,p,b);

eq_displacement_balance(eys,l,p) $ (exec_DR_displac)..
sum(b, INCREASED_DR_LOAD(eys,l,p,b)*duration(l,p,b) ) =e= sum(b, DECREASED_DR_LOAD(eys,l,p,b)*duration(l,p,b));

eq_potency_conservation_limit(eys,l,p,b) $ (exec_DR_reduc and duration(l,p,b))..
CONSERVED_DR_LOAD(eys,l,p,b) =l= conservable_load(eys,l,p,b);

eq_minimal_savings_requirement(eys,l,p) $ (exec_DR_displac)..
sum(b, ( DECREASED_DR_LOAD(eys,l,p,b) )*GAD_energy_only_price(eys,l,p,b)*duration(l,p,b))-sum(b, ( INCREASED_DR_LOAD(eys,l,p,b) )*GAD_energy_only_price(eys,l,p,b)*duration(l,p,b))=l=(1-minimum_required_DR_savings(eys))*sum(b, ( displaceable_load(eys,l,p,b) )*GAD_energy_only_price(eys,l,p,b)*duration(l,p,b));




*-------------------------------------------------------------------------------
*----------------------------- Defining MCP Equations ------------------------
*-------------------------------------------------------------------------------
*Additional Variables:
Negative Variables
LAMBDA_DEM(eys,l,p,b), LAMBDA_RES(eys,l,p), LAMBDA_PUMP_EFF(eys,l), LAMBDA_PUMP_MAX(eys,l), LAMBDA_DISPATCHABLE_RESERVES(eys,l),
LAMBDA_RoR(eys,l,p,b), LAMBDA_FG_USE(eys,l,p,b), LAMBDA_WIND(eys,l,p,b), LAMBDA_ORSR(eys,l,p,b), LAMBDA_PINS_CAPACITY(eys,l,t),
LAMBDA_PINS_AVAILABILITY(eys,t,f,l,p,b),
LAMBDA_MAX_DISPL(eys,l,p,b), LAMBDA_DISPL_BAL(eys,l,p), LAMBDA_CONSERV_LIMIT(eys,l,p,b), LAMBDA_REQ_SAVINGS(eys,l,p)
*LAMBDA_MAX_DISPL2(eys,l,p,b)
 ;


Equations
eq_dL_dPGEN(eys,t,f,l,p,b)
eq_dL_dPPUMPED(eys,l,p,b)
eq_dL_RES(eys,l,p)
eq_dL_PINS(eys,l,t)
eq_dL_dINSTALLED_CAPACITY_LEGEN(eys,l,t)
eq_dL_dINCREASED_DR_LOAD(eys,l,p,b)
eq_dL_dDECREASED_DR_LOAD(eys,l,p,b)
eq_dL_dCONSERVED_DR_LOAD(eys,l,p,b)
eq_dL_dLAMBDA_DEM(eys,l,p,b)
eq_dL_dLAMBDA_DR_DEM(eys,l,p,b)
eq_dL_dLAMBDA_RES(eys,l,p)
eq_dL_dLAMBDA_PUMP_EFF(eys,l)
eq_dL_dLAMBDA_PUMP_MAX(eys,l)
eq_dL_dLAMBDA_DISPATCHABLE_RESERVES(eys,l)
eq_dL_dLAMBDA_RoR(eys,l,p,b)
eq_dL_dLAMBDA_FG_USE(eys,l,p,b)
eq_dL_dLAMBDA_WIND(eys,l,p,b)
eq_dL_dLAMBDA_ORSR(eys,l,p,b)
eq_dL_dLAMBDA_PINS_CAPACITY(eys,l,t)
eq_dL_dLAMBDA_PINS_AVAILABILITY(eys,t,f,l,p,b)
eq_dL_dLAMBDA_MAX_DISPL(eys,l,p,b)
*eq_dL_dLAMBDA_MAX_DISPL2(eys,l,p,b)
eq_dL_dLAMBDA_DISPL_BAL(eys,l,p)
eq_dL_dLAMBDA_CONSERV_LIMIT(eys,l,p,b)
eq_dL_dLAMBDA_REQ_SAVINGS(eys,l,p)
;

eq_dL_dPGEN(eys,t,f,l,p,b)..
         (
           exec_fc*( thermodynamic_efficiency(eys,t,f)*fuel_price(eys,t,f)/1e6 *duration(l,p,b) )$thermal_t(t,f)
         + exec_co2*( co2e_content(f,t)/1e6*co2_price(eys)*duration(l,p,b) )$emitting_t(t,f)
         + exec_vom*( init_oem_vom(eys,t)/1e6*duration(l,p,b) )$t_f(t,f)
         - exec_renew_prem*( duration(l,p,b)*renewable_premium(eys,t)/1e6 )$renewable_premium_t(t,f)
         )
         - LAMBDA_DEM(eys,l,p,b)*
         (
         - 1
         + init_own_cons_gen_pctg
         + tdeo_losses_pctg(l,p,b)
         )$t_f(t,f)
         +  ( - LAMBDA_RES(eys,l,p)*
         (
         duration(l,p,b)
         )
         )$((sameas(t,"Hyd_Res")) and (sameas (f,"na")))
         - ( LAMBDA_PUMP_EFF(eys,l)*
         (
         duration(l,p,b)
         )
         )$((sameas(t,"Pump")) and (sameas (f,"na")))
         + ( - LAMBDA_PUMP_MAX(eys,l)*
         (
         duration(l,p,b)
         )
         )$((sameas(t,"Pump")) and (sameas (f,"na")))
         + ( - LAMBDA_RoR(eys,l,p,b)*
         (
         1
         )
         )$((sameas(t,"Hyd_RoR")) and (sameas (f,"na")))
         + ( - LAMBDA_FG_USE(eys,l,p,b)*
         (
         1-pctg_fueloil_use_on_fg(eys,l)
         )
         )$((sameas(t,"F-G")) and (sameas (f,"Fuel-oil")))
         + ( - LAMBDA_FG_USE(eys,l,p,b)*
         (
         -pctg_fueloil_use_on_fg(eys,l)
         )
         )$((sameas(t,"F-G")) and (sameas (f,"Natural_Gas")))
         + ( - LAMBDA_WIND(eys,l,p,b)*
         (
         1
         )
         )$((sameas(t,"Wind")) and (sameas (f,"na")))
         + ( - LAMBDA_ORSR(eys,l,p,b)*
         (
         1
         )
         )$((sameas(t,"ORSR")) and (sameas (f,"na")))
         - LAMBDA_PINS_AVAILABILITY(eys,t,f,l,p,b)*
         (
         1
         )
         =e= 0;

eq_dL_dPPUMPED(eys,l,p,b)..
         - LAMBDA_DEM(eys,l,p,b)*
         (
         1
         - tdeo_losses_pctg(l,p,b)
         )
         - LAMBDA_PUMP_EFF(eys,l)*
         (
         - pump_efficiency
         )
         =e= 0;


eq_dL_RES(eys,l,p)..
         - LAMBDA_RES(eys,l,p)*
         (
         - 1
         )
         -
         (
         LAMBDA_RES(eys,l,p-1)*
         (
         1
         )
         )$(ORD(p) lt card(p)-1)
         =e= 0;

*eq_hydro_reservoir_level(eys,l,p)..
*sum((b),PGEN_LEGEN(eys,"Hyd_Res","na",l,p,b)*duration(l,p,b))
*- RES_LEGEN(eys,l,p)
*+ RES_LEGEN(eys,l,p+1)$(ORD(p) lt card(p)) + res_init(eys+1,l,"Hyd_Res")$(ORD(p) EQ card(p)) =L= hydro_inflows(eys,l,p,"Hyd_Res");


eq_dL_PINS(eys,l,t)..
         sum((eys_alias)$((ord(eys_alias) ge ord(eys)) and (ord(eys_alias) lt (ord(eys)+power_plant_time_life(t)))),
         + exec_capac*( init_overnight_costs(eys_alias,t) * IDC(t) * capital_recovery_factor(t)/1e3  )
         )
         + sum((eys_alias)$((ord(eys_alias) ge ord(eys)) and (ord(eys_alias) lt (ord(eys)+power_plant_time_life(t)))),
         - LAMBDA_PINS_CAPACITY(eys_alias,l,t)*
         (
         - 1
         )
         )
         =e= 0;

eq_dL_dINSTALLED_CAPACITY_LEGEN(eys,l,t)..
         (
         + exec_oem_labor*( init_oem_fom_labor(eys,l,t) / 1e3 )
         + exec_oem_sc*( init_oem_fom_social_contribution(eys,l,t) / 1e3 )
         + exec_oem_equip*( init_oem_fom_equipments(eys,l,t) / 1e3 )
         )
         + ( - LAMBDA_DISPATCHABLE_RESERVES(eys,l)*
         (
         - 1
         )
         )$t_not_intermittent(t)
         - sum((p,b), ( LAMBDA_WIND(eys,l,p,b)*
         (
         - pgenerated("2005",l,p,b,"Wind") / existent_installed_capacity("2005",l,"Wind")
         )
         ) )$(sameas(t,"Wind"))
         - sum((p,b), ( LAMBDA_ORSR(eys,l,p,b)*
         (
         - pgenerated("2005",l,p,b,"ORSR") / existent_installed_capacity("2005",l,"ORSR")
         )
         ) )$(sameas(t,"ORSR"))
         - LAMBDA_PINS_CAPACITY(eys,l,t)*
         (
         1
         )
         - sum((f,p,b)$t_f(t,f), ( LAMBDA_PINS_AVAILABILITY(eys,t,f,l,p,b)*
         (
         - availability_factor(eys,l,t)*stochastic_adjusted_availability(eys,l,p,b,t)
         ) ) )
         =e= 0;


eq_dL_dINCREASED_DR_LOAD(eys,l,p,b)..
         + ( - LAMBDA_DEM(eys,l,p,b)*
         (
         1
         ) )$(exec_DR_displac)
*         + ( - LAMBDA_MAX_DISPL2(eys,l,p,b)*
*         (
*         1
*         ) )$(exec_DR_displac)
         + ( - LAMBDA_DISPL_BAL(eys,l,p)*
         (
         duration(l,p,b)
         ) )$(exec_DR_displac)
         + ( - LAMBDA_REQ_SAVINGS(eys,l,p)*
         (
         -GAD_energy_only_price(eys,l,p,b)*duration(l,p,b)
         ) )$(exec_DR_displac)
         =e= 0;

eq_dL_dDECREASED_DR_LOAD(eys,l,p,b)..
         + ( - LAMBDA_DEM(eys,l,p,b)*
         (
         - 1
         ) )$(exec_DR_displac)
         + ( - LAMBDA_MAX_DISPL(eys,l,p,b)*
         (
         1
         ) )$(exec_DR_displac)
         + ( - LAMBDA_DISPL_BAL(eys,l,p)*
         (
         - duration(l,p,b)
         ) )$(exec_DR_displac)
         + ( - LAMBDA_REQ_SAVINGS(eys,l,p)*
         (
         GAD_energy_only_price(eys,l,p,b)*duration(l,p,b)
         ) )$(exec_DR_displac)
         =e= 0;


eq_dL_dCONSERVED_DR_LOAD(eys,l,p,b)..
         + ( - LAMBDA_DEM(eys,l,p,b)*
         (
         - 1
         ) )$(exec_DR_reduc)
         + ( - LAMBDA_CONSERV_LIMIT(eys,l,p,b)*
         (
         1
         ) )$(exec_DR_reduc)
         =e= 0;


eq_dL_dLAMBDA_DR_DEM(eys,l,p,b)..
         sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b)) + pimported(eys,l,p,b) - PPUMPED_LEGEN(eys,l,p,b)
         - init_own_cons_gen_pctg*sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b))
         - tdeo_losses_pctg(l,p,b)*( sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b)) + pimported(eys,l,p,b) - PPUMPED_LEGEN(eys,l,p,b) + demand(eys,"Exports",l,p,b)  )
         =e= demand(eys,"households",l,p,b) + sum(sne, demand(eys,sne,l,p,b)) + demand(eys,"Exports",l,p,b)
         + ( INCREASED_DR_LOAD(eys,l,p,b) - DECREASED_DR_LOAD(eys,l,p,b) )$(exec_DR_displac) - ( CONSERVED_DR_LOAD(eys,l,p,b) )$(exec_DR_reduc);


eq_dL_dLAMBDA_DEM(eys,l,p,b)..
         demand(eys,"households",l,p,b) + sum(sne, demand(eys,sne,l,p,b)) + demand(eys,"Exports",l,p,b)
         - sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b)) - pimported(eys,l,p,b) + PPUMPED_LEGEN(eys,l,p,b)
         + init_own_cons_gen_pctg*sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b))
         + tdeo_losses_pctg(l,p,b)*( sum((t,f)$t_f(t,f),PGEN_LEGEN(eys,t,f,l,p,b)) + pimported(eys,l,p,b) - PPUMPED_LEGEN(eys,l,p,b) + demand(eys,"Exports",l,p,b)  )
         =e= 0;

eq_dL_dLAMBDA_RES(eys,l,p)..
         sum((b),PGEN_LEGEN(eys,"Hyd_Res","na",l,p,b)*duration(l,p,b)) - RES_LEGEN(eys,l,p) + RES_LEGEN(eys,l,p+1)$(ORD(p) lt card(p)) + res_init(eys+1,l,"Hyd_Res")$(ORD(p) EQ card(p)) - hydro_inflows(eys,l,p,"Hyd_Res")
         =L= 0;

eq_dL_dLAMBDA_PUMP_EFF(eys,l)..
         sum((p,b), duration(l,p,b)*(PGEN_LEGEN(eys,"Pump","na",l,p,b) - PPUMPED_LEGEN(eys,l,p,b)*pump_efficiency) )
         =L= 0;

eq_dL_dLAMBDA_PUMP_MAX(eys,l)..
         sum((p,b), PGEN_LEGEN(eys,"Pump","na",l,p,b)*duration(l,p,b)) - res_max(eys,l,"Pump")
         =L= 0;

eq_dL_dLAMBDA_DISPATCHABLE_RESERVES(eys,l)..
         not_internittent_coverage_rate * smax((p,b), sum(sam,demand(eys,sam,l,p,b)) ) - sum((t,f)$((t_not_intermittent(t))and (t_f(t,f))), INSTALLED_CAPACITY_LEGEN(eys,l,t))
         =L= 0;

eq_dL_dLAMBDA_RoR(eys,l,p,b)..
         PGEN_LEGEN(eys,"Hyd_RoR","na",l,p,b) - hydro_inflows(eys,l,p,"Hyd_RoR")/sum((b_alias),duration(l,p,b_alias))
         =e= 0;

eq_dL_dLAMBDA_FG_USE(eys,l,p,b)..
         PGEN_LEGEN(eys,"F-G","Fuel-oil",l,p,b)-pctg_fueloil_use_on_fg(eys,l)*sum((f),PGEN_LEGEN(eys,"F-G",f,l,p,b))
         =e= 0;

eq_dL_dLAMBDA_WIND(eys,l,p,b)..
         PGEN_LEGEN(eys,"Wind","na",l,p,b) - pgenerated("2005",l,p,b,"Wind") / existent_installed_capacity("2005",l,"Wind") * INSTALLED_CAPACITY_LEGEN(eys,l,"Wind")
         =e= 0;

eq_dL_dLAMBDA_ORSR(eys,l,p,b)..
         PGEN_LEGEN(eys,"ORSR","na",l,p,b) - pgenerated("2005",l,p,b,"ORSR") / existent_installed_capacity("2005",l,"ORSR") * INSTALLED_CAPACITY_LEGEN(eys,l,"ORSR")
         =e= 0;

eq_dL_dLAMBDA_PINS_CAPACITY(eys,l,t)..
         INSTALLED_CAPACITY_LEGEN(eys,l,t) - ( existent_installed_capacity(eys,l,t)  + sum((eys_alias)$((ord(eys_alias) le ord(eys)) and (ord(eys_alias) gt (ord(eys)-power_plant_time_life(t)))),PINS_LEGEN(eys_alias,l,t)) )
         =e= 0;

eq_dL_dLAMBDA_PINS_AVAILABILITY(eys,t,f,l,p,b)..
         PGEN_LEGEN(eys,t,f,l,p,b) - INSTALLED_CAPACITY_LEGEN(eys,l,t)*availability_factor(eys,l,t)*stochastic_adjusted_availability(eys,l,p,b,t)
         =L=  0;

eq_dL_dLAMBDA_MAX_DISPL(eys,l,p,b)..
         DECREASED_DR_LOAD(eys,l,p,b) - displaceable_load(eys,l,p,b)
         =L=  0;

*eq_dL_dLAMBDA_MAX_DISPL2(eys,l,p,b)..
*         INCREASED_DR_LOAD(eys,l,p,b) - displaceable_load(eys,l,p,b)
*         =L=  0;

eq_dL_dLAMBDA_DISPL_BAL(eys,l,p)..
         sum(b, DECREASED_DR_LOAD(eys,l,p,b)*duration(l,p,b)) - sum(b, INCREASED_DR_LOAD(eys,l,p,b)*duration(l,p,b) )
         =e=  0;

eq_dL_dLAMBDA_CONSERV_LIMIT(eys,l,p,b)..
         CONSERVED_DR_LOAD(eys,l,p,b) - conservable_load(eys,l,p,b)
         =L=  0;

eq_dL_dLAMBDA_REQ_SAVINGS(eys,l,p)..
         sum(b, ( DECREASED_DR_LOAD(eys,l,p,b) )*GAD_energy_only_price(eys,l,p,b)*duration(l,p,b))-sum(b, ( INCREASED_DR_LOAD(eys,l,p,b) )*GAD_energy_only_price(eys,l,p,b)*duration(l,p,b)) - (1-minimum_required_DR_savings(eys))*sum(b, ( displaceable_load(eys,l,p,b) )*GAD_energy_only_price(eys,l,p,b)*duration(l,p,b))
         =L=  0;


*eq_add_1(eys,l,p,b)$(not(Increase_DR_Load_Blocks(l,p,b))..
*       INCREASED_DR_LOAD.fx(eys,l,p,b)=e=0;
*loop(eys, loop((l,p,b)$(not(Increase_DR_Load_Blocks(l,p,b))), INCREASED_DR_LOAD.fx(eys,l,p,b)=0; ); );
*loop(eys, loop((l,p,b)$(not(Decrease_DR_Load_Blocks(l,p,b))), DECREASED_DR_LOAD.fx(eys,l,p,b)=0; ); );

*---------Fixing non used load blocks--------------------
loop((l,p,b)$(not duration(l,p,b)),
PGEN_LEGEN.fx(eys,t,f,l,p,b)=0;
PPUMPED_LEGEN.fx(eys,l,p,b)=0;
VOM_LEGEN.fx(eys,t,f,l,p,b)=0;
FUEL_COST_LEGEN.fx(eys,t,f,l,p,b)=0;
CO2_PAYMENTS_LEGEN.fx(eys,t,f,l,p,b)=0;
);
loop ((t,f)$(not t_f(t,f)),
PGEN_LEGEN.fx(eys,t,f,l,p,b)=0;
FUEL_COST_LEGEN.fx(eys,t,f,l,p,b)=0;
CO2_PAYMENTS_LEGEN.fx(eys,t,f,l,p,b)=0;
VOM_LEGEN.fx(eys,t,f,l,p,b)=0;
);
*fixing PINS to years where even that we choose to construct today the power plant, the construction time make it impossible to them to be available
*I should include exogenously the new capacity installed for these years (between 2010 and 2010+construction time) (take from REE) because these decisions were made before the initial execution year of the model
*I should fix this only after 2010 = substitute ey.val+1 por 2010+1
loop ((ey,eys,t)$((eys.val ge ey.val) and (eys.val le ey.val+construction_time(t))), PINS_LEGEN.fx(eys,l,t)=0; );

*avoiding unecessary displacemnts by setting that displacement only ocur from the bigger demanding hours to the lower demanding hours, avoiding displacements between hours of medium demand
loop(eys, loop((l,p,b)$(not(Increase_DR_Load_Blocks(l,p,b))), INCREASED_DR_LOAD.fx(eys,l,p,b)=0; ); );
*loop(eys, loop((l,p,b)$(not(Decrease_DR_Load_Blocks(l,p,b))), DECREASED_DR_LOAD.fx(eys,l,p,b)=0; ); );

*---------Initialising DR variables
INCREASED_DR_LOAD.l(eys,l,p,b)=0;
DECREASED_DR_LOAD.l(eys,l,p,b)=0;
CONSERVED_DR_LOAD.l(eys,l,p,b)=0;

*-------------------------------------------------------------------------------
*-------------------------------- Defining the Models --------------------------
*-------------------------------------------------------------------------------
MODEL LEGEN_BAU  /
                 eq_Total_generation_cost,
                 eq_demand_balance_BAU,
                 eq_hydro_reservoir_level,
                 eq_pumping_efficiency,
                 eq_maximal_pumping_production,
                 eq_run_of_river_production,
                 eq_fixed_FG_combustible_use,
                 eq_VOM,
                 eq_fuel_cost,
                 eq_CO2_emission_costs,
                 eq_pins_acum,
                 eq_amortization,
                 eq_pins_availability,
                 eq_pgen_wind,
                 eq_pgen_orsr,
                 eq_reserves,
                 eq_objective_function
                 /;

MODEL LEGEN      /
                 eq_Total_generation_cost,
                 eq_hydro_reservoir_level,
                 eq_pumping_efficiency,
                 eq_maximal_pumping_production,
                 eq_run_of_river_production,
                 eq_fixed_FG_combustible_use,
                 eq_VOM,
                 eq_fuel_cost,
                 eq_CO2_emission_costs,
                 eq_pins_acum,
                 eq_amortization,
                 eq_pins_availability,
                 eq_pgen_wind,
                 eq_pgen_orsr,
                 eq_reserves,
                 eq_objective_function
                 eq_demand_balance_case_study,
                 eq_maximum_displacement,
                 eq_displacement_balance,
                 eq_potency_conservation_limit,
                 eq_minimal_savings_requirement
                 /;

MODEL MCPGEN_BAU /
                 eq_dL_dPGEN.PGEN_LEGEN
                 eq_dL_dPPUMPED.PPUMPED_LEGEN
                 eq_dL_RES.RES_LEGEN
                 eq_dL_PINS.PINS_LEGEN
                 eq_dL_dINSTALLED_CAPACITY_LEGEN.INSTALLED_CAPACITY_LEGEN
                 eq_dL_dLAMBDA_DEM.LAMBDA_DEM
                 eq_dL_dLAMBDA_RES.LAMBDA_RES
                 eq_dL_dLAMBDA_PUMP_EFF.LAMBDA_PUMP_EFF
                 eq_dL_dLAMBDA_PUMP_MAX.LAMBDA_PUMP_MAX
                 eq_dL_dLAMBDA_DISPATCHABLE_RESERVES.LAMBDA_DISPATCHABLE_RESERVES
                 eq_dL_dLAMBDA_RoR.LAMBDA_RoR
                 eq_dL_dLAMBDA_FG_USE.LAMBDA_FG_USE
                 eq_dL_dLAMBDA_WIND.LAMBDA_WIND
                 eq_dL_dLAMBDA_ORSR.LAMBDA_ORSR
                 eq_dL_dLAMBDA_PINS_CAPACITY.LAMBDA_PINS_CAPACITY
                 eq_dL_dLAMBDA_PINS_AVAILABILITY.LAMBDA_PINS_AVAILABILITY
                 /;

MODEL MCPGEN     /
                 eq_dL_dPGEN.PGEN_LEGEN
                 eq_dL_dPPUMPED.PPUMPED_LEGEN
                 eq_dL_RES.RES_LEGEN
                 eq_dL_PINS.PINS_LEGEN
                 eq_dL_dINSTALLED_CAPACITY_LEGEN.INSTALLED_CAPACITY_LEGEN
                 eq_dL_dINCREASED_DR_LOAD.INCREASED_DR_LOAD
                 eq_dL_dDECREASED_DR_LOAD.DECREASED_DR_LOAD
                 eq_dL_dCONSERVED_DR_LOAD.CONSERVED_DR_LOAD
                 eq_dL_dLAMBDA_DR_DEM.LAMBDA_DEM
                 eq_dL_dLAMBDA_RES.LAMBDA_RES
                 eq_dL_dLAMBDA_PUMP_EFF.LAMBDA_PUMP_EFF
                 eq_dL_dLAMBDA_PUMP_MAX.LAMBDA_PUMP_MAX
                 eq_dL_dLAMBDA_DISPATCHABLE_RESERVES.LAMBDA_DISPATCHABLE_RESERVES
                 eq_dL_dLAMBDA_RoR.LAMBDA_RoR
                 eq_dL_dLAMBDA_FG_USE.LAMBDA_FG_USE
                 eq_dL_dLAMBDA_WIND.LAMBDA_WIND
                 eq_dL_dLAMBDA_ORSR.LAMBDA_ORSR
                 eq_dL_dLAMBDA_PINS_CAPACITY.LAMBDA_PINS_CAPACITY
                 eq_dL_dLAMBDA_PINS_AVAILABILITY.LAMBDA_PINS_AVAILABILITY
                 eq_dL_dLAMBDA_MAX_DISPL.LAMBDA_MAX_DISPL
                 eq_dL_dLAMBDA_DISPL_BAL.LAMBDA_DISPL_BAL
                 eq_dL_dLAMBDA_CONSERV_LIMIT.LAMBDA_CONSERV_LIMIT
                 eq_dL_dLAMBDA_REQ_SAVINGS.LAMBDA_REQ_SAVINGS
*                 eq_dL_dLAMBDA_MAX_DISPL2.LAMBDA_MAX_DISPL2
                 /;


*-------------------------------------------------------------------------------
LEGEN_BAU.HOLDFIXED= 1;
LEGEN.HOLDFIXED= 1;
MCPGEN_BAU.HOLDFIXED= 1;
MCPGEN.HOLDFIXED= 1;
*OPTION ITERLIM = 0;
option limcol = 1000;
option limrow = 1000;




